from django.contrib.auth import get_user_model


class EmailAuthBackend(object):
    """Docstring for EmailAuthBackend """
    
    def authenticate(self, username=None, password=None):
        """todo: Docstring for authenticate
        
        :param username: arg description
        :type username: type description
        :param password: arg description
        :type password: type description
        :return:
        :rtype:
        """

        user_model = get_user_model()
    
        try:
            user = user_model.objects.get(email=username)

            if user.check_password(password):
                return user
        except user_model.DoesNotExist:
            return None

    # authenticate()

    def get_user(self, user_id):
        """todo: Docstring for get_user
        
        :param user_id: arg description
        :type user_id: type description
        :return:
        :rtype:
        """
        user_model = get_user_model()

        try:
            return user_model.objects.get(pk=user_id)
        except user_model.DoesNotExist:
            return None
    # get_user()
# EmailAuthBackend




import re

from django.db import models
from django.core.mail import send_mail
from django.core import validators
from django.core.signing import Signer
from django.utils import timezone
from django.utils.http import urlquote
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth.models import AbstractUser, AbstractBaseUser, \
    PermissionsMixin, BaseUserManager


class EmailUserManager(BaseUserManager):

    def create_user(self, username, email=None, password=None, **extra_fields):
        """
        Creates and saves a User with the given username, email and password.
        email is required, username is not.
        """
        now = timezone.now()
        if not email:
            raise ValueError('The given email must be set')
        email = EmailUserManager.normalize_email(email)
        user = self.model(username=username, email=email,
                          is_staff=False, is_active=True, is_superuser=False,
                          last_login=now, date_joined=now, **extra_fields)

        user.set_password(password)
        user.save(using=self._db)
        return user

    def create_superuser(self, username, email, password, **extra_fields):
        u = self.create_user(username, email, password, **extra_fields)
        u.is_staff = True
        u.is_active = True
        u.is_superuser = True
        u.save(using=self._db)
        return u
#EmailUserManager


class EmailUserModel(AbstractBaseUser, PermissionsMixin):
    """Docstring for EmailUserModel """

    date_of_birth = models.DateField(
        help_text=(""),
    )

    date_of_birth = models.DateField(
        help_text=(""),
    )

    username = models.CharField(
        _('username'), max_length=30, unique=True,
        blank=True,
        help_text=_('Required. 30 characters or fewer. Letters, numbers and '
                    '@/./+/-/_ characters'),
        validators=[
            validators.RegexValidator(
                re.compile('^[\w.@+-]+$'), _('Enter a valid username.'), 'invalid')
        ])
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=30, blank=True)
    email = models.EmailField(_('email address'), blank=False, unique=True)
    is_staff = models.BooleanField(
        _('staff status'), default=False,
        help_text=_('Designates whether the user can log into this admin '
                    'site.'))
    is_active = models.BooleanField(
        _('active'), default=True,
        help_text=_('Designates whether this user should be treated as '
                    'active. Unselect this instead of deleting accounts.'))
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)

    objects = EmailUserManager()

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')

    USERNAME_FIELD = 'email'
    REQUIRED_FIELDS = ['username', 'date_of_birth']

    def get_absolute_url(self):
        return "/users/%s/" % urlquote(self.username)

    def get_full_name(self):
        """
        Returns the first_name plus the last_name, with a space in between.
        """
        full_name = '%s %s' % (self.first_name, self.last_name)
        return full_name.strip()

    def get_short_name(self):
        "Returns the short name for the user."
        return self.first_name

    def email_user(self, subject, message, from_email=None):
        """
        Sends an email to this User.
        """
        send_mail(subject, message, from_email, [self.email])

    def __unicode__(self):
        """todo: Docstring for __unicode__
        :return:
        :rtype:
        """
        return self.get_full_name()
    #__unicode__()
#EmailUserModel



from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin
from django.contrib.auth.forms import UserCreationForm, UserChangeForm

from .models import EmailUserModel

# We need to override the base User creation and change forms
# because they are hard coded to use the User model, we just
# need to make them use our EmailUserModel model

# Override the forms:
class EmailUserCreationForm(UserCreationForm):
    class Meta:
        model = get_user_model()
        fields = UserCreationForm.Meta.fields

    def clean_username(self):
        # Since User.username is unique, this check is redundant,
        # but it sets a nicer error message than the ORM. See #13147.
        username = self.cleaned_data["username"]
        try:
            self._meta.model._default_manager.get(username=username)
        except self._meta.model.DoesNotExist:
            return username
        raise forms.ValidationError(self.error_messages['duplicate_username'])


class EmailUserChangeForm(UserChangeForm):
    """Docstring for EmailUserChangeForm """
    class Meta:
        model = get_user_model()
        fields = UserChangeForm.Meta.fields


# Add the forms to our Admin class
class EmailUserAdmin(UserAdmin):

    form = EmailUserChangeForm
    add_form = EmailUserCreationForm

admin.site.register(EmailUserModel, EmailUserAdmin)



#!/usr/bin/env python
# encoding: utf-8

import setuptools
from setuptools import setup, find_packages

import django_email_user_model


setup(name='django-email-user-model',
      version=django_email_user_model.__version__,
      description="Django custom user model that enable E-Mail as username",
      author="Jeff Buttars",
      author_email="jeffbuttars@gmail.com",
      packages=find_packages(),
      license='MIT',
      package_dir={'django_email_user_model': 'django_email_user_model'},
      install_requires=[
          'Django',
      ],
      # data_files=[
      #     ('/etc/init.d', ['conf/init.d/afile']),
      # ],
      )
